local addonName, addon = ...
setfenv(1, addon)

local max, min = max, min

local GetNetStats, PERFORMANCEBAR_UPDATE_INTERVAL = GetNetStats, PERFORMANCEBAR_UPDATE_INTERVAL or 10

--[[-----------------------------------------------------------------------------
Support
-------------------------------------------------------------------------------]]
local SetChecked

local function GetName(self)
	return frame:GetName()
end

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function OnClick(self, button)
	if button == 'LeftButton' then
		local toggle = _G[self.toggle]
		if type(toggle) == 'function' then
			pcall(toggle)
		else
			pcall(ToggleFrame, toggle)
		end
	elseif button == 'RightButton' and Config then
		Config()
	else
		TitanPanelButton_OnClick(frame, button)
	end
	UpdateMicroButtons()
end

local function OnEnter(self, motion)
	TitanPanelButton_OnEnter(frame)
	if self.isMoving or not TitanPanelGetVar('ToolTipsShown') or (TitanPanelGetVar('HideTipsInCombat') and InCombatLockdown()) then return end
	GameTooltip:SetOwner(frame, 'ANCHOR_NONE')
	GameTooltip:SetPoint('TOP', frame, 'BOTTOM')
	GameTooltip:SetText(self.title, 1, 1, 1)
	local keyBind = self.keyBind and GetBindingKey(self.keyBind)
	if keyBind then
		GameTooltip:AppendText(NORMAL_FONT_COLOR_CODE .. " (" .. keyBind .. ")")
	end
	if GetCVarBool('showNewbieTips') then
		GameTooltip:AddLine(self.tooltip, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true)
	end
	if self.tooltipExtend then
		pcall(self.tooltipExtend)
	end
	GameTooltip:Show()
end

local function OnLeave(self, motion)
	TitanPanelButton_OnLeave(frame)
	if GameTooltip:IsOwned(frame) then
		GameTooltip:Hide()
	end
end

local function OnDragStart(self, button)
	if settings.unlock and not IsModifierKeyDown() then
		GameTooltip:Hide()
		self.isMoving, self.x, self.y = self:GetFrameStrata(), self:GetCenter()
		self:ClearAllPoints()
		self:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', self.x, self.y)
		self:SetFrameStrata('TOOLTIP')
		self:StartMoving()
	else
		frame:GetScript('OnDragStart')(frame)
	end
end

local function OnDragStop(self, button)
	if self.isMoving then
		self:StopMovingOrSizing()
		self:SetFrameStrata(self.isMoving)
		self.isMoving = nil
		local old, x, y, adj, iMax = self.order, self:GetCenter()
		if x < frame:GetLeft() + BUTTON_SIZE * (old - 0.5) + settings.spacing * (old - 1) then
			adj, iMax = -1, 1
		else
			adj, iMax = 1, #menuOrder
		end
		local new = iMax
		for index = old + adj, iMax, adj do
			if (buttons[menuOrder[index]]:GetCenter() - x) * adj > 0 then
				new = index - adj
				break
			end
		end
		AdjustMenuOrder(self.id, new)
	else
		frame:GetScript('OnDragStop')(frame)
	end
	if GetMouseFocus() == self then
		OnEnter(self)
	end
end

local latency, overlay, timer = 0, frame:CreateTexture(nil, 'OVERLAY'), PERFORMANCEBAR_UPDATE_INTERVAL
local function OnUpdate(self, elapsed)															-- Menu button only
	timer = timer + elapsed
	if timer < PERFORMANCEBAR_UPDATE_INTERVAL then return end
	timer = 0
	_, _, latency = GetNetStats()
	overlay:SetVertexColor(min(latency / LATENCY_MID_RANGE, 1), 1 - min(max(latency - LATENCY_MID_RANGE, 0) / LATENCY_MID_RANGE, 1), 0)
end

--[[-----------------------------------------------------------------------------
Create buttons
-------------------------------------------------------------------------------]]
local defaultCoords, menuCoords = { 5/32, 27/32, 31/64, 58/64 }, { 4/32, 28/32, 33/64, 53/64 }

local function CreateButton(id, name, title, toggle, tooltip, keyBind, texCoords, texture)
	local button = CreateFrame('CheckButton', nil, frame)
	button.texture = button:CreateTexture(nil, 'ARTWORK')
	button.texture:SetAllPoints(button)
	button:SetCheckedTexture([[Interface\Buttons\CheckButtonHilight]])
	button:SetHighlightTexture([[Interface\Buttons\ButtonHilight-Square]])
	button:SetPushedTexture([[Interface\Buttons\ButtonHilight-Square]])
	button:GetPushedTexture():SetBlendMode('ADD')

	if texture then
		button.texture:SetTexture(texture)
	elseif texture ~= false then
		button.texture:SetTexture(([[Interface\Buttons\UI-MicroButton-%s-Up]]):format(name))
	end
	button.texture:SetTexCoord(unpack(texCoords or defaultCoords))

	if not SetChecked then
		SetChecked = button.SetChecked
	end

	button.GetName = GetName
	button.SetChecked = DoNothing
	button:SetButtonState('NORMAL', false)
	button:SetScript('OnDragStart', OnDragStart)
	button:SetScript('OnDragStop', OnDragStop)
	button:SetScript('OnEnter', OnEnter)
	button:SetScript('OnLeave', OnLeave)
	button:SetScript('OnClick', OnClick)
	button:EnableMouse(true)
	button:SetMovable(true)
	button:RegisterForClicks('AnyUp')
	button:RegisterForDrag('LeftButton')
	button:Show()

	button.id, button.name, button.title, button.toggle, button.tooltip, button.keyBind = id, name, title, toggle, tooltip, keyBind
	buttons[id], buttons[name] = button, button
	return button
end

local button = CreateButton(1, 'Character', CHARACTER_BUTTON, 'CharacterFrame', NEWBIE_TOOLTIP_CHARACTER, 'TOGGLECHARACTER0', { 0.1, 0.9, 0.0666, 0.9 }, false)
button.background = button:CreateTexture(nil, 'BACKGROUND')
button.background:SetAllPoints(button)
button.background:SetTexture([[Interface\BUTTONS\WHITE8X8]])
button.background:SetVertexColor(0, 0, 0, 1)
button:SetScript('OnEvent', function(self)
	SetPortraitTexture(self.texture, 'player')
end)
button:RegisterEvent('PLAYER_ENTERING_WORLD')
button:RegisterEvent('UNIT_PORTRAIT_UPDATE')

CreateButton(2, 'Spellbook', SPELLBOOK_ABILITIES_BUTTON, 'SpellBookFrame', NEWBIE_TOOLTIP_SPELLBOOK, 'TOGGLESPELLBOOK')
CreateButton(3, 'Talents', TALENTS_BUTTON, 'ToggleTalentFrame', NEWBIE_TOOLTIP_TALENTS, 'TOGGLETALENTS')
CreateButton(4, 'Achievement', ACHIEVEMENT_BUTTON, 'ToggleAchievementFrame', NEWBIE_TOOLTIP_ACHIEVEMENT, 'TOGGLEACHIEVEMENT')
CreateButton(5, 'Quest', QUESTLOG_BUTTON, 'QuestLogFrame', NEWBIE_TOOLTIP_QUESTLOG, 'TOGGLEQUESTLOG')
CreateButton(6, 'Socials', SOCIAL_BUTTON, 'FriendsFrame', NEWBIE_TOOLTIP_SOCIAL, 'TOGGLESOCIAL')

button = CreateButton(7, 'PVP', PLAYER_V_PLAYER, 'TogglePVPFrame', NEWBIE_TOOLTIP_PVP, 'TOGGLECHARACTER4', { 4/64, 38/64, 2/64, 38/64 }, false)
button.background = button:CreateTexture(nil, 'BACKGROUND')
button.background:SetAllPoints(button)
button.background:SetTexture([[Interface\BUTTONS\WHITE8X8]])
button.background:SetVertexColor(0, 0, 0, 1)
button:SetScript('OnEvent', function(self, event)
	self.texture:SetTexture([[Interface\TargetingFrame\UI-PVP-]] .. UnitFactionGroup('player'))
	self:UnregisterEvent(event)
	self:SetScript('OnEvent', nil)
end)
button:RegisterEvent('PLAYER_ENTERING_WORLD')

CreateButton(8, 'LFG', DUNGEONS_BUTTON, 'ToggleLFDParentFrame', NEWBIE_TOOLTIP_LFGPARENT, 'TOGGLELFGPARENT')

button = CreateButton(9, 'MainMenu', MAINMENU_BUTTON, 'GameMenuFrame', NEWBIE_TOOLTIP_MAINMENU, 'TOGGLEGAMEMENU', menuCoords, ([[Interface\AddOns\%s\MainMenu]]):format(addonName))
overlay:SetParent(button)
overlay:SetAllPoints(button)
overlay:SetTexture(([[Interface\AddOns\%s\LatencyOverlay]]):format(addonName))
overlay:SetTexCoord(unpack(menuCoords))
button:SetScript('OnUpdate', OnUpdate)
button.tooltipExtend = function()
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(MAINMENUBAR_LATENCY_LABEL:format(latency), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	GameTooltip:AddLine(MAINMENUBAR_FPS_LABEL:format(GetFramerate()), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
end

CreateButton(10, 'Help', HELP_BUTTON, 'ToggleHelpFrame', NEWBIE_TOOLTIP_HELP)

--[[-----------------------------------------------------------------------------
Hooks
-------------------------------------------------------------------------------]]
local watchedFrames = {
	['CharacterFrame'] = buttons[1],
	['SpellBookFrame'] = buttons[2],
	['PlayerTalentFrame'] = buttons[3],
	['AchievementFrame'] = buttons[4],
	['QuestLogFrame'] = buttons[5],
	['FriendsFrame'] = buttons[6],
	['PVPParentFrame'] = buttons[7],
	['LFDParentFrame'] = buttons[8],
	['GameMenuFrame'] = buttons[9],
	['AudioOptionsFrame'] = buttons[9],
	['InterfaceOptionsFrame'] = buttons[9],
	['KeyBindingFrame'] = buttons[9],
--	['MacroFrame'] = buttons[9],
	['VideoOptionsFrame'] = buttons[9],
	['HelpFrame'] = buttons[10]
}

local function UpdateMicroButtons()
	for id = 1, #buttons do
		SetChecked(buttons[id], false)
	end
	for name, button in pairs(watchedFrames) do
		if _G[name] and _G[name]:IsVisible() then
			SetChecked(button, true)
		end
	end
end

hooksecurefunc('HideUIPanel', function(frame)
	if frame and not watchedFrames[frame:GetName()] then return end
	UpdateMicroButtons()
end)

hooksecurefunc('ShowUIPanel', function(frame)
	frame = frame and watchedFrames[frame:GetName()]
	if not frame then return end
	SetChecked(frame, true)
end)

hooksecurefunc('UpdateMicroButtons', UpdateMicroButtons)

--[[-----------------------------------------------------------------------------
Initialize
-------------------------------------------------------------------------------]]
menuOrderDefault = { }
for id = 1, #buttons do
	menuOrderDefault[id] = id
end
