local addonName, addon = ...
setmetatable(addon, { __index = _G })
setfenv(1, addon)

buttons = { }
BUTTON_SIZE = 16

--[[-----------------------------------------------------------------------------
Titan button
-------------------------------------------------------------------------------]]
frame = CreateFrame('Button', 'TitanPanelMicroMenuButton', UIParent, 'TitanPanelButtonTemplate')
frame:SetFrameStrata('FULLSCREEN')
frame:SetToplevel(true)
frame:SetScript('OnClick', function(self, button)
	if button == 'RightButton' and Config then
		Config()
	end
end)

frame.registry = {
	id = "MicroMenu",
	category = 'Interface',
	version = GetAddOnMetadata(addonName, 'Version')
}

--[[-----------------------------------------------------------------------------
Helper functions
-------------------------------------------------------------------------------]]
local function table_copy(source, destination)
	if type(destination) == 'table' then
		wipe(destination)
	else
		destination = { }
	end
	for key, value in pairs(source) do
		destination[key] = value
	end
	return destination
end

--[[-----------------------------------------------------------------------------
Global to addon
-------------------------------------------------------------------------------]]
function AdjustMenuOrder(id, new)
	local old = buttons[id].order
	local adj = old > new and -1 or 1
	for index = old, new - adj, adj do
		menuOrder[index] = menuOrder[index + adj]
	end
	menuOrder[new] = id
	CopyMenuOrderToButtons()
	RefreshDisplay()
	return new ~= old
end

function CopyMenuOrderToButtons()
	for id = 1, #buttons do
		buttons[id].order = nil
	end
	for index = 1, #menuOrder do
		buttons[menuOrder[index]].order = index
	end
end

function RefreshDisplay()
	local button, offsetX
	for id = 1, #buttons do
		button = buttons[id]
		if button.order then
			offsetX = button.order - 1
			button:SetHeight(BUTTON_SIZE)
			button:SetWidth(BUTTON_SIZE)
			button:ClearAllPoints()
			button:SetPoint('LEFT', frame, 'LEFT', (BUTTON_SIZE + settings.spacing) * offsetX, 0)
			button:Show()
		else
			button:Hide()
		end
	end
	frame:SetHeight(TitanPanelBarButton:GetHeight())
	frame:SetWidth(BUTTON_SIZE * #menuOrder + settings.spacing * (#menuOrder - 1))
end

function ResetSettings()
	local order = settings.menuOrder
	wipe(settings)
	settings.latency		= 600
	settings.menuCount	= #menuOrderDefault
	settings.menuOrder	= table_copy(menuOrderDefault, order)
	settings.spacing		= 3
end

--[[-----------------------------------------------------------------------------
Initialize
-------------------------------------------------------------------------------]]
frame:SetScript('OnEvent', function(self, event, name)
	if name ~= addonName then return end
	frame:UnregisterEvent(event)
	frame:SetScript('OnEvent', nil)

	settings = _G[addonName .. "Settings"]
	if type(settings) ~= 'table' or type(settings.menuOrder) ~= 'table' or settings.menuCount ~= #menuOrderDefault then
		settings = { }
		_G[addonName .. "Settings"] = settings
		ResetSettings()
	end

	LATENCY_MID_RANGE = settings.latency / 2
	menuOrder = settings.menuOrder
	CopyMenuOrderToButtons()
	RefreshDisplay()
end)
frame:RegisterEvent('ADDON_LOADED')

local LOA = LibStub('LibOptionsAssist-1.0', true)
if not (LOA and select(2, GetAddOnInfo(addonName .. '_Config'))) then return end	-- Make sure config support exists

Config = LOA:AddEntry(addonName:gsub("_", " - "), nil, function()
	local config = addonName .. '_Config'
	addon.name, _G[config] = addonName, addon
	LibStub('LibOptionsAssist-1.0'):LoadModule(config)
	addon.name, _G[config] = nil, nil
end)
