if not _G[...] then return end
local addon = _G[...]
local addonName = addon.name
setfenv(1, addon)

local MAX_BUTTON_SPACING	= 10

--[[-----------------------------------------------------------------------------
Local to module
-------------------------------------------------------------------------------]]
local function Get(info)
	return settings[info[#info]]
end

local function Set(info, value)
	settings[info[#info]] = value
end

local function SetWithRefresh(info, value)
	settings[info[#info]] = value
	RefreshDisplay()
end

local function GetButtonStatus(info)
	return buttons[info[#info]].order
end

local function SetButtonStatus(info, value)
	local id = buttons[info[#info]].id
	if not value then
		for index = 1, #menuOrder do
			if menuOrder[index] == id then
				tremove(menuOrder, index)
				break
			end
		end
		id = #menuOrder < 1 and 1 or nil
	end
	if id then
		menuOrder[#menuOrder + 1] = id
	end
	CopyMenuOrderToButtons()
	RefreshDisplay()
end

local function DisableOption(info)
	local field = info[#info]
	if field == "latency" then
		return not buttons['MainMenu'].order
	end
end

local function SetLatencyRange(_, value)
	settings.latency = value
	LATENCY_MID_RANGE = value / 2
	buttons['MainMenu']:GetScript('OnUpdate')(buttons['MainMenu'], 100)
end

--[[-----------------------------------------------------------------------------
Options
-------------------------------------------------------------------------------]]
local options = {
	type = 'group',
	get = GetButtonStatus,
	set = SetButtonStatus,
	args = {
		unlock = {
			type = 'toggle',
			order = 100,
			name = "Unlock",
			desc = "Unlock the buttons to change their order.",
			get = Get,
			set = Set
		},
		spacing = {
			type = 'range',
			order = 101,
			name = "Spacing",
			desc = "Set the spacing between the buttons.",
			min = 0, max = MAX_BUTTON_SPACING, step = 1,
			get = Get,
			set = SetWithRefresh
		},
		latency = {
			type = 'range',
			order = 102,
			name = "Latency Indicator",
			desc = "The computer screen on the Game Menu button changes color to indicate latency. This value determines the point at which it will be red.",
			min = 100, max = 1000, step = 10,
			disabled = DisableOption,
			get = Get,
			set = SetLatencyRange
		},
		reset = {
			type = 'execute',
			order = 103,
			name = "Reset",
			desc = "Click to reset settings to their defaults.",
			func = function()
				ResetSettings()
				CopyMenuOrderToButtons()
				SetLatencyRange(_, settings.latency)
				RefreshDisplay()
			end
		}
	}
}

local half, option = ceil(#buttons / 2)
for id, button in ipairs(buttons) do
	option = {
		type = 'toggle',
		name = button.title,
		desc = "Show the " .. button.title .. " button.",
	}
	if id <= half then
		option.order = (id - 1) * 2 + 1
	else
		option.order = (id - half) * 2
	end
	options.args[button.name] = option
end

--[[-----------------------------------------------------------------------------
Initialize
-------------------------------------------------------------------------------]]
LibStub('AceConfigRegistry-3.0'):RegisterOptionsTable(addonName, options)

Config:AssignOptions(addonName)
Config:SetDesc(("These options allow you to change the appearance and behavior of %s."):format(addonName))
Config:SetInfo(("Version: %s"):format(GetAddOnMetadata(addonName, 'Version')))
